@@ -11,8 +11,8 @@ the upper layer:
 * self.dotfiles
 * self.profiles
 * self.actions
-* self.trans_r
-* self.trans_w
+* self.trans_install
+* self.trans_update
 * self.variables
 
 Additionally a few methods are exported.
@@ -26,7 +26,11 @@ import io
 from copy import deepcopy
 from itertools import chain
 from ruamel.yaml import YAML as yaml
-import toml
+try:
+    import tomllib
+except ImportError:
+    import tomli as tomllib
+import tomli_w
 
 # local imports
 from dotdrop.version import __version__ as VERSION
@@ -46,9 +50,11 @@ class CfgYaml:
     key_dotfiles = 'dotfiles'
     key_profiles = 'profiles'
     key_actions = 'actions'
-    old_key_trans_r = 'trans'
-    key_trans_r = 'trans_read'
-    key_trans_w = 'trans_write'
+    old_key_trans = 'trans'
+    old_key_trans_r = 'trans_read'
+    old_key_trans_w = 'trans_write'
+    key_trans_install = 'trans_install'
+    key_trans_update = 'trans_update'
     key_variables = 'variables'
     key_dvariables = 'dynvariables'
     key_uvariables = 'uservariables'
@@ -109,7 +115,8 @@ class CfgYaml:
     top_entries = [key_dotfiles, key_settings, key_profiles]
 
     def __init__(self, path, profile=None, addprofiles=None,
-                 reloading=False, debug=False, imported_configs=None):
+                 reloading=False, debug=False, imported_configs=None,
+                 fail_on_error=True):
         """
         config parser
         @path: config file path
@@ -142,8 +149,8 @@ class CfgYaml:
         self.dotfiles = {}
         self.profiles = {}
         self.actions = {}
-        self.trans_r = {}
-        self.trans_w = {}
+        self.trans_install = {}
+        self.trans_update = {}
         self.variables = {}
 
         if not os.path.exists(self._path):
@@ -162,7 +169,7 @@ class CfgYaml:
         # live patch deprecated entries
         self._fix_deprecated(self._yaml_dict)
         # validate content
-        self._validate(self._yaml_dict)
+        self._validate(self._yaml_dict, fail_on_error)
 
         ##################################################
         # parse the config and variables
@@ -188,7 +195,7 @@ class CfgYaml:
         self._add_variables(dvariables, template=False)
 
         # now template variables and dynvariables from the same pool
-        self._rec_resolve_variables(self.variables)
+        self.variables = self._rec_resolve_variables(self.variables)
         # and execute dvariables
         # since this is done after recursively resolving variables
         # and dynvariables this means that variables referencing
@@ -244,10 +251,10 @@ class CfgYaml:
         self.dotfiles = self._parse_blk_dotfiles(self._yaml_dict)
         # parse the "actions" block
         self.actions = self._parse_blk_actions(self._yaml_dict)
-        # parse the "trans_r" block
-        self.trans_r = self._parse_blk_trans_r(self._yaml_dict)
-        # parse the "trans_w" block
-        self.trans_w = self._parse_blk_trans_w(self._yaml_dict)
+        # parse the "trans_install" block
+        self.trans_install = self._parse_blk_trans_install(self._yaml_dict)
+        # parse the "trans_update" block
+        self.trans_update = self._parse_blk_trans_update(self._yaml_dict)
 
         ##################################################
         # import elements
@@ -423,7 +430,7 @@ class CfgYaml:
         return True
 
     def add_dotfile(self, key, src, dst, link, chmod=None,
-                    trans_r_key=None, trans_w_key=None):
+                    trans_install_key=None, trans_update_key=None):
         """add a new dotfile"""
         if key in self.dotfiles.keys():
             return False
@@ -432,9 +439,10 @@ class CfgYaml:
             self._dbg(f'new dotfile src: {src}')
             self._dbg(f'new dotfile dst: {dst}')
             self._dbg(f'new dotfile link: {link}')
-            self._dbg(f'new dotfile chmod: {chmod}')
-            self._dbg(f'new dotfile trans_r: {trans_r_key}')
-            self._dbg(f'new dotfile trans_w: {trans_w_key}')
+            if chmod:
+                self._dbg(f'new dotfile chmod: {chmod:o}')
+            self._dbg(f'new dotfile trans_install: {trans_install_key}')
+            self._dbg(f'new dotfile trans_update: {trans_update_key}')
 
         # create the dotfile dict
         df_dict = {
@@ -451,11 +459,11 @@ class CfgYaml:
         if chmod:
             df_dict[self.key_dotfile_chmod] = str(format(chmod, 'o'))
 
-        # trans_r/trans_w
-        if trans_r_key:
-            df_dict[self.key_trans_r] = str(trans_r_key)
-        if trans_w_key:
-            df_dict[self.key_trans_w] = str(trans_w_key)
+        # trans_install/trans_update
+        if trans_install_key:
+            df_dict[self.key_trans_install] = str(trans_install_key)
+        if trans_update_key:
+            df_dict[self.key_trans_update] = str(trans_update_key)
 
         if self._debug:
             self._dbg(f'dotfile dict: {df_dict}')
@@ -613,30 +621,25 @@ class CfgYaml:
             self._debug_dict('actions block', actions)
         return actions
 
-    def _parse_blk_trans_r(self, dic):
-        """parse the "trans_r" block"""
-        key = self.key_trans_r
-        if self.old_key_trans_r in dic:
-            msg = '\"trans\" is deprecated, please use \"trans_read\"'
-            self._log.warn(msg)
-            dic[self.key_trans_r] = dic[self.old_key_trans_r]
-            del dic[self.old_key_trans_r]
-        trans_r = self._get_entry(dic, key, mandatory=False)
-        if trans_r:
-            trans_r = trans_r.copy()
+    def _parse_blk_trans_install(self, dic):
+        """parse the "trans_install" block"""
+        trans_install = self._get_entry(dic, self.key_trans_install,
+                                        mandatory=False)
+        if trans_install:
+            trans_install = trans_install.copy()
         if self._debug:
-            self._debug_dict('trans_r block', trans_r)
-        return trans_r
+            self._debug_dict('trans_install block', trans_install)
+        return trans_install
 
-    def _parse_blk_trans_w(self, dic):
-        """parse the "trans_w" block"""
-        trans_w = self._get_entry(dic, self.key_trans_w,
-                                  mandatory=False)
-        if trans_w:
-            trans_w = trans_w.copy()
+    def _parse_blk_trans_update(self, dic):
+        """parse the "trans_update" block"""
+        trans_update = self._get_entry(dic, self.key_trans_update,
+                                       mandatory=False)
+        if trans_update:
+            trans_update = trans_update.copy()
         if self._debug:
-            self._debug_dict('trans_w block', trans_w)
-        return trans_w
+            self._debug_dict('trans_update block', trans_update)
+        return trans_update
 
     def _parse_blk_variables(self, dic):
         """parse the "variables" block"""
@@ -812,6 +815,7 @@ class CfgYaml:
         if not dotfiles:
             return dotfiles
         new = {}
+
         for k, val in dotfiles.items():
             if self.key_dotfile_src not in val:
                 # add 'src' as key' if not present
@@ -820,14 +824,6 @@ class CfgYaml:
             else:
                 new[k] = val
 
-            if self.old_key_trans_r in val:
-                # fix deprecated trans key
-                msg = f'{k} \"trans\" is deprecated, please use \"trans_read\"'
-                self._log.warn(msg)
-                val[self.key_trans_r] = val[self.old_key_trans_r]
-                del val[self.old_key_trans_r]
-                new[k] = val
-
             if self.key_dotfile_link not in val:
                 # apply link value if undefined
                 value = self.settings[self.key_settings_link_dotfile_default]
@@ -873,7 +869,7 @@ class CfgYaml:
         self._redefine_templater()
         if template:
             # rec resolve variables with new ones
-            self._rec_resolve_variables(self.variables)
+            self.variables = self._rec_resolve_variables(self.variables)
         if shell and new:
             # shell exec
             self._shell_exec_dvars(self.variables, keys=new.keys())
@@ -1038,7 +1034,7 @@ class CfgYaml:
                                     mandatory=False)
 
             merged = self._merge_dict(dvar, var)
-            self._rec_resolve_variables(merged)
+            merged = self._rec_resolve_variables(merged)
             if dvar.keys():
                 self._shell_exec_dvars(merged, keys=dvar.keys())
             self._clear_profile_vars(merged)
@@ -1085,7 +1081,8 @@ class CfgYaml:
         sub = CfgYaml(path, profile=self._profile,
                       addprofiles=self._inc_profiles,
                       debug=self._debug,
-                      imported_configs=self.imported_configs)
+                      imported_configs=self.imported_configs,
+                      fail_on_error=False)
 
         # settings are ignored from external file
         # except for filter_file and func_file
@@ -1103,8 +1100,10 @@ class CfgYaml:
         self.profiles = self._merge_dict(self.profiles, sub.profiles,
                                          deep=True)
         self.actions = self._merge_dict(self.actions, sub.actions)
-        self.trans_r = self._merge_dict(self.trans_r, sub.trans_r)
-        self.trans_w = self._merge_dict(self.trans_w, sub.trans_w)
+        self.trans_install = self._merge_dict(self.trans_install,
+                                              sub.trans_install)
+        self.trans_update = self._merge_dict(self.trans_update,
+                                             sub.trans_update)
         self._clear_profile_vars(sub.variables)
 
         self.imported_configs.append(path)
@@ -1184,6 +1183,54 @@ class CfgYaml:
             return
         self._fix_deprecated_link_by_default(yamldict)
         self._fix_deprecated_dotfile_link(yamldict)
+        self._fix_deprecated_trans(yamldict)
+
+    def _fix_deprecated_trans_in_dict(self, yamldic):
+        # trans -> trans_install
+        old_key = self.old_key_trans
+        new_key = self.key_trans_install
+        if old_key in yamldic:
+            yamldic[old_key] = yamldic[new_key]
+            del yamldic[old_key]
+            msg = f'deprecated \"{old_key}\", '
+            msg += f', updated to {new_key}\"'
+            self._log.warn(msg)
+            self._dirty = True
+            self._dirty_deprecated = True
+
+        # trans_read -> trans_install
+        old_key = self.old_key_trans_r
+        new_key = self.key_trans_install
+        if old_key in yamldic:
+            yamldic[new_key] = yamldic[old_key]
+            del yamldic[old_key]
+            warn = f'deprecated \"{old_key}\"'
+            warn += f', updated to \"{new_key}\"'
+            self._log.warn(warn)
+            self._dirty = True
+            self._dirty_deprecated = True
+
+        # trans_write -> trans_update
+        old_key = self.old_key_trans_w
+        new_key = self.key_trans_update
+        if old_key in yamldic:
+            yamldic[new_key] = yamldic[old_key]
+            del yamldic[old_key]
+            warn = f'deprecated \"{old_key}\"'
+            warn += f', updated to \"{new_key}\"'
+            self._log.warn(warn)
+            self._dirty = True
+            self._dirty_deprecated = True
+
+    def _fix_deprecated_trans(self, yamldict):
+        """fix deprecated trans key"""
+        # top ones
+        self._fix_deprecated_trans_in_dict(yamldict)
+        # dotfiles ones
+        if self.key_dotfiles in yamldict and yamldict[self.key_dotfiles]:
+            config = yamldict[self.key_dotfiles]
+            for _, val in config.items():
+                self._fix_deprecated_trans_in_dict(val)
 
     def _fix_deprecated_link_by_default(self, yamldict):
         """fix deprecated link_by_default"""
@@ -1292,9 +1339,11 @@ class CfgYaml:
             self._dbg(f'format: {self._config_format}')
         return content
 
-    def _validate(self, yamldict):
+    def _validate(self, yamldict, fail_on_error):
         """validate entries"""
         if not yamldict:
+            if fail_on_error:
+                raise YamlException('empty config file')
             return
 
         # check top entries
@@ -1307,21 +1356,24 @@ class CfgYaml:
         # check link_dotfile_default
         if self.key_settings not in yamldict:
             # no configs top entry
+            if fail_on_error:
+                raise YamlException(f'no \"{self.key_settings}\" key found')
             return
         if not yamldict[self.key_settings]:
             # configs empty
+            if fail_on_error:
+                raise YamlException(f'empty \"{self.key_settings}\" key')
             return
 
         # check settings values
         settings = yamldict[self.key_settings]
-        if self.key_settings_link_dotfile_default not in settings:
-            return
-        val = settings[self.key_settings_link_dotfile_default]
-        if val not in self.allowed_link_val:
-            err = f'bad link value: {val}'
-            self._log.err(err)
-            self._log.err(f'allowed: {self.allowed_link_val}')
-            raise YamlException(f'config content error: {err}')
+        if self.key_settings_link_dotfile_default in settings:
+            val = settings[self.key_settings_link_dotfile_default]
+            if val not in self.allowed_link_val:
+                err = f'bad link value: {val}'
+                self._log.err(err)
+                self._log.err(f'allowed: {self.allowed_link_val}')
+                raise YamlException(f'config content error: {err}')
 
     @classmethod
     def _yaml_load(cls, path):
@@ -1345,7 +1397,7 @@ class CfgYaml:
         """load from toml"""
         with open(path, 'r', encoding='utf8') as file:
             data = file.read()
-        content = toml.loads(data)
+        content = tomllib.loads(data)
         # handle inexistent dotfiles/profiles
         # since toml doesn't have a nul/nil/null/none
         if cls.key_dotfiles not in content:
@@ -1375,7 +1427,7 @@ class CfgYaml:
     @classmethod
     def __toml_dump(cls, content, file):
         """dump to toml"""
-        toml.dump(content, file)
+        file.write(tomli_w.dumps(content))
 
     ########################################################
     # templating
@@ -1398,11 +1450,11 @@ class CfgYaml:
         template an item using the templategen
         will raise an exception if template failed and exc_if_fail
         """
-        if not Templategen.var_is_template(item):
+        if not Templategen.string_is_template(item):
             return item
         try:
             val = item
-            while Templategen.var_is_template(val):
+            while Templategen.string_is_template(val):
                 val = self._tmpl.generate_string(val)
         except UndefinedException as exc:
             if exc_if_fail:
@@ -1502,14 +1554,21 @@ class CfgYaml:
         templ = Templategen(variables=var,
                             func_file=func_files,
                             filter_file=filter_files)
+        newvars = variables.copy()
         for k in variables.keys():
             val = variables[k]
             while Templategen.var_is_template(val):
-                val = templ.generate_string(val)
-                variables[k] = val
-                templ.update_variables(variables)
-        if variables is self.variables:
+                val = templ.generate_string_or_dict(val)
+                if isinstance(val, dict):
+                    for sub in val:
+                        subkey = f'{k}.{sub}'
+                        newvars[subkey] = val[sub]
+                else:
+                    newvars[k] = val
+                templ.update_variables(newvars)
+        if newvars is self.variables:
             self._redefine_templater()
+        return newvars
 
     def _get_profile_included_vars(self):
         """
@@ -1657,10 +1716,13 @@ class CfgYaml:
                 if k not in final:
                     final[k] = []
                 final[k] += val
+            elif isinstance(val, str):
+                final[k] = val
             else:
                 # don't know how to handle
                 err = 'unable to merge'
                 raise YamlException(err)
+
         return final
 
     @classmethod
@@ -1748,6 +1810,7 @@ class CfgYaml:
             if self._debug:
                 self._dbg(f'{k}: `{val}` -> {out}')
             dic[k] = out
+            self._debug_dict('dynvars after', dic)
 
     @classmethod
     def _check_minversion(cls, minversion):
@@ -1773,8 +1836,8 @@ class CfgYaml:
         self._debug_dict('entry dotfiles', self.dotfiles)
         self._debug_dict('entry profiles', self.profiles)
         self._debug_dict('entry actions', self.actions)
-        self._debug_dict('entry trans_r', self.trans_r)
-        self._debug_dict('entry trans_w', self.trans_w)
+        self._debug_dict('entry trans_install', self.trans_install)
+        self._debug_dict('entry trans_update', self.trans_update)
         self._debug_dict('entry variables', self.variables)
 
     def _debug_dict(self, title, elems):
