@@ -6,11 +6,11 @@ handle the comparison of two dotfiles
 """
 
 import os
-import filecmp
 
 # local imports
 from dotdrop.logger import Logger
-from dotdrop.utils import must_ignore, uniq_list, diff, \
+from dotdrop.ftree import FTreeDir
+from dotdrop.utils import must_ignore, diff, \
     get_file_perm
 
 
@@ -22,6 +22,7 @@ class Comparator:
         """constructor
         @diff_cmd: diff command to use
         @debug: enable debug
+        @ignore_missing_in_dotdrop: ignore missing files in dotdrop
         """
         self.diff_cmd = diff_cmd
         self.debug = debug
@@ -38,9 +39,27 @@ class Comparator:
             ignore = []
         local_path = os.path.expanduser(local_path)
         deployed_path = os.path.expanduser(deployed_path)
-        self.log.dbg(f'comparing {local_path} and {deployed_path}')
+
+        self.log.dbg(f'comparing \"{local_path}\" and \"{deployed_path}\"')
         self.log.dbg(f'ignore pattern(s): {ignore}')
 
+        return self._compare(local_path, deployed_path,
+                             ignore=ignore, mode=mode,
+                             recurse=True)
+
+    def _compare(self, local_path, deployed_path,
+                 ignore=None, mode=None,
+                 recurse=False):
+        if not ignore:
+            ignore = []
+
+        # test existence
+        if not os.path.exists(local_path):
+            return f'=> \"{local_path}\" does not exist on destination\n'
+        if not self.ignore_missing_in_dotdrop:
+            if not os.path.exists(deployed_path):
+                return f'=> \"{deployed_path}\" does not exist in dotdrop\n'
+
         # test type of file
         if os.path.isdir(local_path) and not os.path.isdir(deployed_path):
             ret = f'\"{local_path}\" is a dir'
@@ -51,7 +70,7 @@ class Comparator:
             ret += f' while \"{deployed_path}\" is a dir\n'
             return ret
 
-        # test content
+        # is a file
         if not os.path.isdir(local_path):
             self.log.dbg(f'{local_path} is a file')
             ret = self._comp_file(local_path, deployed_path, ignore)
@@ -59,9 +78,11 @@ class Comparator:
                 ret = self._comp_mode(local_path, deployed_path, mode=mode)
             return ret
 
-        self.log.dbg(f'{local_path} is a directory')
-
-        ret = self._comp_dir(local_path, deployed_path, ignore)
+        # is a directory
+        self.log.dbg(f'\"{local_path}\" is a directory')
+        ret = ''
+        if recurse:
+            ret = self._comp_dir(local_path, deployed_path, ignore)
         if not ret:
             ret = self._comp_mode(local_path, deployed_path, mode=mode)
         return ret
@@ -93,86 +114,60 @@ class Comparator:
                                debug=self.debug):
             self.log.dbg(f'ignoring diff {local_path} and {deployed_path}')
             return ''
-        return self._diff(local_path, deployed_path)
+        return self._diff(local_path, deployed_path, header=True)
 
     def _comp_dir(self, local_path, deployed_path, ignore):
         """compare a directory"""
         self.log.dbg(f'compare directory {local_path} with {deployed_path}')
         if not os.path.exists(deployed_path):
             return ''
-        if (self.ignore_missing_in_dotdrop and not
-                os.path.exists(local_path)) \
-                or must_ignore([local_path, deployed_path], ignore,
-                               debug=self.debug):
+        ign_missing = self.ignore_missing_in_dotdrop and not \
+            os.path.exists(local_path)
+        paths = [local_path, deployed_path]
+        must_ign = must_ignore(paths,
+                               ignore,
+                               debug=self.debug)
+        if ign_missing or must_ign:
             self.log.dbg(f'ignoring diff {local_path} and {deployed_path}')
             return ''
         if not os.path.isdir(deployed_path):
             return f'\"{deployed_path}\" is a file\n'
 
-        return self._compare_dirs(local_path, deployed_path, ignore)
+        return self._compare_dirs2(local_path, deployed_path, ignore)
 
-    def _compare_dirs(self, local_path, deployed_path, ignore):
+    def _compare_dirs2(self, local_path, deployed_path, ignore):
         """compare directories"""
-        self.log.dbg(f'compare {local_path} and {deployed_path}')
+        self.log.dbg(f'compare dirs {local_path} and {deployed_path}')
         ret = []
-        comp = filecmp.dircmp(local_path, deployed_path)
 
-        # handle files only in deployed dir
-        self.log.dbg(f'files only in deployed dir: {comp.left_only}')
-        for i in comp.left_only:
-            if self.ignore_missing_in_dotdrop or \
-               must_ignore([os.path.join(local_path, i)],
-                           ignore, debug=self.debug):
+        local_tree = FTreeDir(local_path, ignores=ignore, debug=self.debug)
+        deploy_tree = FTreeDir(deployed_path, ignores=ignore, debug=self.debug)
+        lonly, ronly, common = local_tree.compare(deploy_tree)
+
+        for i in lonly:
+            path = os.path.join(local_path, i)
+            if os.path.isdir(path):
+                # ignore dir
                 continue
-            ret.append(f'=> \"{i}\" does not exist on destination\n')
+            ret.append(f'=> \"{path}\" does not exist on destination\n')
+        if not self.ignore_missing_in_dotdrop:
+            for i in ronly:
+                path = os.path.join(deployed_path, i)
+                if os.path.isdir(path):
+                    # ignore dir
+                    continue
+                ret.append(f'=> \"{path}\" does not exist in dotdrop\n')
 
-        # handle files only in dotpath dir
-        self.log.dbg(f'files only in dotpath dir: {comp.right_only}')
-        for i in comp.right_only:
-            if must_ignore([os.path.join(deployed_path, i)],
-                           ignore, debug=self.debug):
-                continue
-
-            if not self.ignore_missing_in_dotdrop:
-                ret.append(f'=> \"{i}\" does not exist in dotdrop\n')
-
-        # same local_path and deployed_path but different type
-        funny = comp.common_funny
-        self.log.dbg(f'files with different types: {funny}')
-        for i in funny:
+        # test for content difference
+        # and mode difference
+        self.log.dbg(f'common files {common}')
+        for i in common:
             source_file = os.path.join(local_path, i)
             deployed_file = os.path.join(deployed_path, i)
-            if self.ignore_missing_in_dotdrop and \
-                    not os.path.exists(source_file):
-                continue
-            if must_ignore([source_file, deployed_file],
-                           ignore, debug=self.debug):
-                continue
-            short = os.path.basename(source_file)
-            # file vs dir
-            ret.append(f'=> different type: \"{short}\"\n')
-
-        # content is different
-        funny = comp.diff_files
-        funny.extend(comp.funny_files)
-        funny = uniq_list(funny)
-        self.log.dbg(f'files with different content: {funny}')
-        for i in funny:
-            source_file = os.path.join(local_path, i)
-            deployed_file = os.path.join(deployed_path, i)
-            if self.ignore_missing_in_dotdrop and \
-                    not os.path.exists(source_file):
-                continue
-            if must_ignore([source_file, deployed_file],
-                           ignore, debug=self.debug):
-                continue
-            ret.append(self._diff(source_file, deployed_file, header=True))
-
-        # recursively compare subdirs
-        for i in comp.common_dirs:
-            sublocal_path = os.path.join(local_path, i)
-            subdeployed_path = os.path.join(deployed_path, i)
-            ret.extend(self._comp_dir(sublocal_path, subdeployed_path, ignore))
+            subret = self._compare(source_file, deployed_file,
+                                   ignore=None, mode=None,
+                                   recurse=False)
+            ret.extend(subret)
 
         return ''.join(ret)
 
@@ -180,7 +175,7 @@ class Comparator:
         """diff two files"""
         out = diff(modified=local_path, original=deployed_path,
                    diff_cmd=self.diff_cmd, debug=self.debug)
-        if header:
+        if header and out:
             lshort = os.path.basename(local_path)
             out = f'=> diff \"{lshort}\":\n{out}'
         return out
