@@ -16,12 +16,13 @@ from dotdrop.options import Options
 from dotdrop.logger import Logger
 from dotdrop.templategen import Templategen
 from dotdrop.installer import Installer
+from dotdrop.uninstaller import Uninstaller
 from dotdrop.updater import Updater
 from dotdrop.comparator import Comparator
 from dotdrop.importer import Importer
 from dotdrop.utils import get_tmpdir, removepath, \
-    uniq_list, patch_ignores, dependencies_met, \
-    adapt_workers, check_version, pivot_path
+    uniq_list, ignores_to_absolute, dependencies_met, \
+    adapt_workers, check_version, pivot_path, dir_empty
 from dotdrop.linktypes import LinkTypes
 from dotdrop.exceptions import YamlException, \
     UndefinedException, UnmetDependency, \
@@ -100,7 +101,8 @@ def _dotfile_compare(opts, dotfile, tmp):
                      dry=opts.dry, base=opts.dotpath,
                      workdir=opts.workdir, debug=opts.debug,
                      backup_suffix=opts.install_backup_suffix,
-                     diff_cmd=opts.diff_command)
+                     diff_cmd=opts.diff_command,
+                     force_chmod=True)
     comp = Comparator(diff_cmd=opts.diff_command, debug=opts.debug,
                       ignore_missing_in_dotdrop=ignore_missing_in_dotdrop)
 
@@ -120,9 +122,10 @@ def _dotfile_compare(opts, dotfile, tmp):
 
     # apply transformation
     tmpsrc = None
-    if dotfile.trans_r:
+    if dotfile.trans_install:
         LOG.dbg('applying transformation before comparing')
-        tmpsrc = apply_trans(opts.dotpath, dotfile, templ, debug=opts.debug)
+        tmpsrc = apply_install_trans(opts.dotpath, dotfile,
+                                     templ, debug=opts.debug)
         if not tmpsrc:
             # could not apply trans
             return False
@@ -138,12 +141,13 @@ def _dotfile_compare(opts, dotfile, tmp):
         return True
 
     ignores = list(set(opts.compare_ignore + dotfile.cmpignore))
-    ignores = patch_ignores(ignores, dotfile.dst, debug=opts.debug)
+    ignores = ignores_to_absolute(ignores, [dotfile.dst, dotfile.src],
+                                  debug=opts.debug)
 
     insttmp = None
-    if dotfile.template and Templategen.is_template(src,
-                                                    ignore=ignores,
-                                                    debug=opts.debug):
+    if dotfile.template and \
+        Templategen.path_is_template(src,
+                                     debug=opts.debug):
         # install dotfile to temporary dir for compare
         ret, err, insttmp = inst.install_to_temp(templ, tmp, src, dotfile.dst,
                                                  is_template=True,
@@ -165,11 +169,13 @@ def _dotfile_compare(opts, dotfile, tmp):
     if tmpsrc:
         tmpsrc = os.path.join(opts.dotpath, tmpsrc)
         if os.path.exists(tmpsrc):
-            removepath(tmpsrc, LOG)
+            # ignore error
+            removepath(tmpsrc, logger=LOG)
 
     # clean tmp template dotfile if any
     if insttmp and os.path.exists(insttmp):
-        removepath(insttmp, LOG)
+        # ignore error
+        removepath(insttmp, logger=LOG)
 
     if diff != '':
         # print diff results
@@ -215,11 +221,11 @@ def _dotfile_install(opts, dotfile, tmpdir=None):
     LOG.dbg(dotfile.prt())
 
     ignores = list(set(opts.install_ignore + dotfile.instignore))
-    ignores = patch_ignores(ignores, dotfile.dst, debug=opts.debug)
+    ignores = ignores_to_absolute(ignores, [dotfile.dst, dotfile.src],
+                                  debug=opts.debug)
 
-    is_template = dotfile.template and Templategen.is_template(
+    is_template = dotfile.template and Templategen.path_is_template(
         dotfile.src,
-        ignore=ignores,
     )
     if hasattr(dotfile, 'link') and dotfile.link in (
         LinkTypes.LINK, LinkTypes.LINK_CHILDREN,
@@ -231,34 +237,31 @@ def _dotfile_install(opts, dotfile, tmpdir=None):
                                 actionexec=pre_actions_exec,
                                 is_template=is_template,
                                 ignore=ignores,
-                                chmod=dotfile.chmod,
-                                force_chmod=opts.install_force_chmod)
+                                chmod=dotfile.chmod)
     else:
         # nolink
         src = dotfile.src
         tmp = None
-        if dotfile.trans_r:
-            tmp = apply_trans(opts.dotpath, dotfile, templ, debug=opts.debug)
+        if dotfile.trans_install:
+            tmp = apply_install_trans(opts.dotpath, dotfile,
+                                      templ, debug=opts.debug)
             if not tmp:
                 return False, dotfile.key, None
             src = tmp
         # make sure to re-evaluate if is template
-        is_template = dotfile.template and Templategen.is_template(
-            src,
-            ignore=ignores,
-        )
+        is_template = dotfile.template and Templategen.path_is_template(src)
         ret, err = inst.install(templ, src, dotfile.dst,
                                 LinkTypes.NOLINK,
                                 actionexec=pre_actions_exec,
                                 noempty=dotfile.noempty,
                                 ignore=ignores,
                                 is_template=is_template,
-                                chmod=dotfile.chmod,
-                                force_chmod=opts.install_force_chmod)
+                                chmod=dotfile.chmod)
         if tmp:
             tmp = os.path.join(opts.dotpath, tmp)
             if os.path.exists(tmp):
-                removepath(tmp, LOG)
+                # ignore error
+                removepath(tmp, logger=LOG)
 
     # check result of installation
     if ret:
@@ -322,6 +325,7 @@ def cmd_install(opts):
         for root, _, files in os.walk(opts.workdir):
             for file in files:
                 fpath = os.path.join(root, file)
+                # ignore error
                 removepath(fpath, logger=LOG)
 
     # execute profile pre-action
@@ -389,7 +393,7 @@ def _workdir_enum(opts):
         if dotfile.link == LinkTypes.NOLINK:
             # ignore not link files
             continue
-        if not Templategen.is_template(src):
+        if not Templategen.path_is_template(src):
             # ignore not template
             continue
         newpath = pivot_path(dotfile.dst, opts.workdir,
@@ -533,15 +537,16 @@ def cmd_importer(opts):
                         dry=opts.dry, safe=opts.safe,
                         debug=opts.debug,
                         keepdot=opts.keepdot,
-                        ignore=opts.import_ignore)
+                        ignore=opts.import_ignore,
+                        forcekey=opts.import_force_key)
 
     for path in paths:
         tmpret = importer.import_path(path,
                                       import_as=opts.import_as,
                                       import_link=opts.import_link,
                                       import_mode=opts.import_mode,
-                                      import_transw=opts.import_transw,
-                                      import_transr=opts.import_transr)
+                                      trans_install=opts.import_trans_install,
+                                      trans_update=opts.import_trans_update)
         if tmpret < 0:
             ret = False
         elif tmpret > 0:
@@ -582,7 +587,7 @@ def cmd_files(opts):
     for dotfile in opts.dotfiles:
         if opts.files_templateonly:
             src = os.path.join(opts.dotpath, dotfile.src)
-            if not Templategen.is_template(src):
+            if not Templategen.path_is_template(src):
                 continue
         if opts.files_grepable:
             fmt = f'{dotfile.key},'
@@ -612,7 +617,7 @@ def cmd_detail(opts):
     dotfiles = opts.dotfiles
     if opts.detail_keys:
         # filtered dotfiles to install
-        uniq = uniq_list(opts.details_keys)
+        uniq = uniq_list(opts.detail_keys)
         dotfiles = [d for d in dotfiles if d.key in uniq]
     LOG.emph(f'dotfiles details for profile \"{opts.profile}\":\n')
     for dotfile in dotfiles:
@@ -620,6 +625,47 @@ def cmd_detail(opts):
     LOG.log('')
 
 
+def cmd_uninstall(opts):
+    """uninstall"""
+    dotfiles = opts.dotfiles
+    keys = opts.uninstall_key
+
+    if keys:
+        # uninstall only specific keys for this profile
+        dotfiles = []
+        for key in uniq_list(keys):
+            dotfile = opts.conf.get_dotfile(key)
+            if dotfile:
+                dotfiles.append(dotfile)
+
+    if not dotfiles:
+        msg = f'no dotfile to uninstall for this profile (\"{opts.profile}\")'
+        LOG.warn(msg)
+        return False
+
+    if opts.debug:
+        lfs = [k.key for k in dotfiles]
+        LOG.dbg(f'dotfiles registered for uninstall: {lfs}')
+
+    uninst = Uninstaller(base=opts.dotpath,
+                         workdir=opts.workdir,
+                         dry=opts.dry,
+                         safe=opts.safe,
+                         debug=opts.debug,
+                         backup_suffix=opts.install_backup_suffix)
+    uninstalled = 0
+    for dotf in dotfiles:
+        res, msg = uninst.uninstall(dotf.src,
+                                    dotf.dst,
+                                    dotf.link)
+        if not res:
+            LOG.err(msg)
+            continue
+        uninstalled += 1
+    LOG.log(f'\n{uninstalled} dotfile(s) uninstalled.')
+    return True
+
+
 def cmd_remove(opts):
     """remove dotfile from dotpath and from config"""
     paths = opts.remove_path
@@ -680,16 +726,17 @@ def cmd_remove(opts):
 
             # remove dotfile from dotpath
             dtpath = os.path.join(opts.dotpath, dotfile.src)
-            removepath(dtpath, LOG)
+            removepath(dtpath, logger=LOG)
             # remove empty directory
             parent = os.path.dirname(dtpath)
             # remove any empty parent up to dotpath
             while parent != opts.dotpath:
-                if os.path.isdir(parent) and not os.listdir(parent):
+                if os.path.isdir(parent) and dir_empty(parent):
                     msg = f'Remove empty dir \"{parent}\"'
                     if opts.safe and not LOG.ask(msg):
                         break
-                    removepath(parent, LOG)
+                    if not removepath(parent, logger=LOG):
+                        LOG.warn(f'unable to remove {parent}')
                 parent = os.path.dirname(parent)
             removed.append(dotfile)
 
@@ -722,7 +769,9 @@ def _get_install_installer(opts, tmpdir=None):
                      totemp=tmpdir,
                      showdiff=opts.install_showdiff,
                      backup_suffix=opts.install_backup_suffix,
-                     diff_cmd=opts.diff_command)
+                     diff_cmd=opts.diff_command,
+                     remove_existing_in_dir=opts.install_remove_existing,
+                     force_chmod=opts.install_force_chmod)
     return inst
 
 
@@ -746,7 +795,7 @@ def _detail(dotpath, dotfile):
     path = os.path.join(dotpath, os.path.expanduser(dotfile.src))
     if not os.path.isdir(path):
         template = 'no'
-        if dotfile.template and Templategen.is_template(path):
+        if dotfile.template and Templategen.path_is_template(path):
             template = 'yes'
         LOG.sub(f'{path} (template:{template})')
     else:
@@ -754,7 +803,7 @@ def _detail(dotpath, dotfile):
             for file in files:
                 fpath = os.path.join(root, file)
                 template = 'no'
-                if dotfile.template and Templategen.is_template(fpath):
+                if dotfile.template and Templategen.path_is_template(fpath):
                     template = 'yes'
                 LOG.sub(f'{fpath} (template:{template})')
 
@@ -774,22 +823,24 @@ def _select(selections, dotfiles):
     return selected
 
 
-def apply_trans(dotpath, dotfile, templater, debug=False):
+def apply_install_trans(dotpath, dotfile, templater, debug=False):
     """
-    apply the read transformation to the dotfile
+    apply the install transformation to the dotfile
     return None if fails and new source if succeed
     """
     src = dotfile.src
     new_src = f'{src}.{TRANS_SUFFIX}'
-    trans = dotfile.trans_r
-    LOG.dbg(f'executing transformation: {trans}')
+    trans = dotfile.trans_install
+    LOG.dbg(f'executing install transformation: {trans}')
     srcpath = os.path.join(dotpath, src)
     temp = os.path.join(dotpath, new_src)
     if not trans.transform(srcpath, temp, templater=templater, debug=debug):
-        msg = f'transformation \"{trans.key}\" failed for {dotfile.key}'
+        msg = f'install transformation \"{trans.key}\"'
+        msg += f'failed for {dotfile.key}'
         LOG.err(msg)
         if new_src and os.path.exists(new_src):
-            removepath(new_src, LOG)
+            # ignore error
+            removepath(new_src, logger=LOG)
         return None
     return new_src
 
@@ -829,7 +880,8 @@ def _exec_command(opts):
             tmp = get_tmpdir()
             ret = cmd_compare(opts, tmp)
             # clean tmp directory
-            removepath(tmp, LOG)
+            # ignore any error
+            removepath(tmp, logger=LOG)
 
         elif opts.cmd_import:
             # import dotfile(s)
@@ -855,6 +907,12 @@ def _exec_command(opts):
             LOG.dbg(f'running cmd: {command}')
             cmd_remove(opts)
 
+        elif opts.cmd_uninstall:
+            # uninstall dotfile
+            command = 'uninstall'
+            LOG.dbg(f'running cmd: {command}')
+            cmd_uninstall(opts)
+
     except UndefinedException as exc:
         LOG.err(exc)
         ret = False
@@ -878,16 +936,16 @@ def main():
     try:
         opts = Options()
     except YamlException as exc:
-        LOG.err(f'error (yaml): {exc}')
+        LOG.err(f'yaml error: {exc}')
         return False
     except ConfigException as exc:
-        LOG.err(f'error (config): {exc}')
+        LOG.err(f'config error: {exc}')
         return False
     except UndefinedException as exc:
-        LOG.err(f'error (deps): {exc}')
+        LOG.err(f'dependencies error: {exc}')
         return False
     except OptionsException as exc:
-        LOG.err(f'error (options): {exc}')
+        LOG.err(f'options error: {exc}')
         return False
 
     if opts.debug:
@@ -902,6 +960,7 @@ def main():
     ret, command = _exec_command(opts)
     cmd_time = time.time() - time0
 
+    opts.debug_command()
     LOG.dbg(f'done executing command \"{command}\"')
     LOG.dbg(f'options loaded in {options_time}')
     LOG.dbg(f'command executed in {cmd_time}')
