@@ -10,6 +10,7 @@ import shutil
 
 # local imports
 from dotdrop.logger import Logger
+from dotdrop.ftree import FTreeDir
 from dotdrop.utils import strip_home, get_default_file_perms, \
     get_file_perm, get_umask, must_ignore, \
     get_unique_tmp_name, removepath
@@ -24,10 +25,10 @@ class Importer:
 
     def __init__(self, profile, conf, dotpath, diff_cmd,
                  variables, dry=False, safe=True, debug=False,
-                 keepdot=True, ignore=None):
+                 keepdot=True, ignore=None, forcekey=None):
         """constructor
         @profile: the selected profile
-        @conf: configuration manager
+        @conf: configuration manager (CfgAggregator)
         @dotpath: dotfiles dotpath
         @diff_cmd: diff command to use
         @variables: dictionary of variables for the templates
@@ -36,6 +37,8 @@ class Importer:
         @debug: enable debug
         @keepdot: keep dot prefix
         @ignore: patterns to ignore when importing
+        @forcekey: force the use of a specific dotfile key
+
         This may raise UndefinedException
         """
         self.profile = profile
@@ -49,20 +52,34 @@ class Importer:
         self.safe = safe
         self.debug = debug
         self.keepdot = keepdot
-        self.ignore = ignore or []
+        self.ignore = []
+        self.forcekey = forcekey
+        self.log = Logger(debug=self.debug)
+
+        # patch ignore patterns
+        if ignore:
+            for ign in ignore:
+                if ign.startswith('!'):
+                    self.ignore.append(ign)
+                    continue
+                if ign.startswith('*/'):
+                    self.ignore.append(ign)
+                    continue
+                newign = f'*/{ign}'
+                self.log.dbg(f'patching ignore {ign} to {newign}')
+                self.ignore.append(newign)
 
         self.templater = Templategen(variables=self.variables,
                                      base=self.dotpath,
                                      debug=self.debug)
 
         self.umask = get_umask()
-        self.log = Logger(debug=self.debug)
 
     def import_path(self, path, import_as=None,
                     import_link=LinkTypes.NOLINK,
                     import_mode=False,
-                    import_transw="",
-                    import_transr=""):
+                    trans_install="",
+                    trans_update=""):
         """
         import a dotfile pointed by path
         returns:
@@ -76,24 +93,25 @@ class Importer:
             self.log.err(f'\"{path}\" does not exist, ignored!')
             return -1
 
-        # check transw if any
-        trans_write = None
-        trans_read = None
-        if import_transw:
-            trans_write = self.conf.get_trans_w(import_transw)
-        if import_transr:
-            trans_read = self.conf.get_trans_r(import_transr)
+        # check trans_update if any
+        tinstall = None
+        tupdate = None
+        if trans_install:
+            tinstall = self.conf.get_trans_install(trans_install)
+        if trans_update:
+            tupdate = self.conf.get_trans_update(trans_update)
 
         return self._import(path, import_as=import_as,
                             import_link=import_link,
                             import_mode=import_mode,
-                            trans_write=trans_write,
-                            trans_read=trans_read)
+                            trans_update=tupdate,
+                            trans_install=tinstall)
 
     def _import(self, path, import_as=None,
                 import_link=LinkTypes.NOLINK,
                 import_mode=False,
-                trans_write=None, trans_read=None):
+                trans_install=None,
+                trans_update=None):
         """
         import path
         returns:
@@ -103,38 +121,39 @@ class Importer:
         """
 
         # normalize path
-        dst = path.rstrip(os.sep)
-        dst = os.path.abspath(dst)
+        infspath = path.rstrip(os.sep)
+        infspath = os.path.abspath(infspath)
 
         # test if must be ignored
-        if self._ignore(dst):
+        if self._ignore(infspath):
             return 0
 
         # ask confirmation for symlinks
         if self.safe:
-            realdst = os.path.realpath(dst)
-            if dst != realdst:
-                msg = f'\"{dst}\" is a symlink, dereference it and continue?'
+            realdst = os.path.realpath(infspath)
+            if infspath != realdst:
+                msg = f'\"{infspath}\" is a symlink, '
+                msg += 'dereference it and continue?'
                 if not self.log.ask(msg):
                     return 0
 
         # create src path
-        src = strip_home(dst)
+        indotpath = strip_home(infspath)
         if import_as:
             # handle import as
-            src = os.path.expanduser(import_as)
-            src = src.rstrip(os.sep)
-            src = os.path.abspath(src)
-            src = strip_home(src)
-            self.log.dbg(f'import src for {dst} as {src}')
+            indotpath = os.path.expanduser(import_as)
+            indotpath = indotpath.rstrip(os.sep)
+            indotpath = os.path.abspath(indotpath)
+            indotpath = strip_home(indotpath)
+            self.log.dbg(f'import src for {infspath} as {indotpath}')
         # with or without dot prefix
         strip = '.' + os.sep
         if self.keepdot:
             strip = os.sep
-        src = src.lstrip(strip)
+        indotpath = indotpath.lstrip(strip)
 
         # get the permission
-        perm = get_file_perm(dst)
+        perm = get_file_perm(infspath)
 
         # get the link attribute
         linktype = import_link
@@ -143,29 +162,32 @@ class Importer:
             self.log.err(f'importing \"{path}\" failed!')
             return -1
 
-        if self._already_exists(src, dst):
+        if self._already_exists(indotpath, infspath):
             return -1
 
-        self.log.dbg(f'import dotfile: src:{src} dst:{dst}')
-
-        if not self._import_file(src, dst, trans_write=trans_write):
+        self.log.dbg(f'import dotfile: src:{indotpath} dst:{infspath}')
+        if not self._import_to_dotpath(indotpath,
+                                       infspath,
+                                       trans_update=trans_update):
+            self.log.dbg('import files failed')
             return -1
 
-        return self._import_in_config(path, src, dst, perm, linktype,
+        return self._import_in_config(path, indotpath,
+                                      infspath, perm, linktype,
                                       import_mode,
-                                      trans_w=trans_write,
-                                      trans_r=trans_read)
+                                      trans_update=trans_update,
+                                      trans_install=trans_install)
 
     def _import_in_config(self, path, src, dst, perm,
                           linktype, import_mode,
-                          trans_r=None, trans_w=None):
+                          trans_install=None,
+                          trans_update=None):
         """
         import path
         returns:
             1: 1 dotfile imported
             0: ignored
         """
-
         # handle file mode
         chmod = None
         dflperm = get_default_file_perms(dst, self.umask)
@@ -177,8 +199,9 @@ class Importer:
 
         # add file to config file
         retconf = self.conf.new_dotfile(src, dst, linktype, chmod=chmod,
-                                        trans_read=trans_r,
-                                        trans_write=trans_w)
+                                        trans_install=trans_install,
+                                        trans_update=trans_update,
+                                        forcekey=self.forcekey)
         if not retconf:
             self.log.warn(f'\"{path}\" ignored during import')
             return 0
@@ -188,7 +211,7 @@ class Importer:
 
     def _check_existing_dotfile(self, src, dst):
         """
-        check if a dotfile in the dotpath
+        check if a dotfile file in the dotpath
         already exists for this src
         """
         if not os.path.exists(src):
@@ -209,69 +232,62 @@ class Importer:
             self.log.dbg('will overwrite existing file')
         return True
 
-    def _import_file(self, src, dst, trans_write=None):
+    def _import_to_dotpath(self, in_dotpath, in_fs, trans_update=None):
         """
-        prepare hierarchy for dotfile in dotpath
-        and copy file
-        src is file in dotpath
-        dst is file on filesystem
+        copy files to dotpath
         """
-        srcf = os.path.join(self.dotpath, src)
-        srcfd = os.path.dirname(srcf)
-
-        # check if must be ignored
-        if self._ignore(srcf) or self._ignore(srcfd):
-            return False
+        in_dotpath_abs = os.path.join(self.dotpath, in_dotpath)
 
         # check we are not overwritting
-        if not self._check_existing_dotfile(srcf, dst):
+        if not self._check_existing_dotfile(in_dotpath_abs, in_fs):
+            self.log.dbg(f'{in_dotpath_abs} exits already')
             return False
 
-        # create directory hierarchy
-        if self.dry:
-            cmd = f'mkdir -p {srcfd}'
-            self.log.dry(f'would run: {cmd}')
-        else:
-            try:
-                os.makedirs(srcfd, exist_ok=True)
-            except OSError:
-                self.log.err(f'importing \"{dst}\" failed!')
-                return False
-
         # import the file
         if self.dry:
-            self.log.dry(f'would copy {dst} to {srcf}')
-        else:
-            # apply trans_w
-            dst = self._apply_trans_w(dst, trans_write)
-            if not dst:
-                # transformation failed
-                return False
-            # copy the file to the dotpath
-            try:
-                if os.path.isdir(dst):
-                    if os.path.exists(srcf):
-                        shutil.rmtree(srcf)
-                    ign = shutil.ignore_patterns(*self.ignore)
-                    shutil.copytree(dst, srcf,
-                                    copy_function=self._cp,
-                                    ignore=ign)
-                else:
-                    shutil.copy2(dst, srcf)
-            except shutil.Error as exc:
-                src = exc.args[0][0][0]
-                why = exc.args[0][0][2]
-                self.log.err(f'importing \"{src}\" failed: {why}')
+            self.log.dry(f'would copy {in_fs} to {in_dotpath_abs}')
+            return True
 
+        # apply trans_update
+        in_fs = self._apply_trans_update(in_fs, trans_update)
+        if not in_fs:
+            # transformation failed
+            self.log.dbg(f"trans failed: {in_fs}")
+            return False
+
+        if not os.path.isdir(in_fs):
+            # handle file
+            self._import_file_to_dotpath(in_fs, in_dotpath_abs)
+
+        # handle dir and get a list of all files to import
+        fstree = FTreeDir(in_fs,
+                          ignores=self.ignore,
+                          debug=self.debug)
+
+        self.log.dbg(f'{len(fstree.get_entries())} files to import')
+        for entry in fstree.get_entries():
+            self.log.dbg(f"importing {entry}...")
+            src = os.path.join(in_fs, entry)
+            rel_src = os.path.relpath(entry, in_fs)
+            dst = os.path.join(in_dotpath_abs, rel_src)
+            if os.path.isdir(src):
+                # we do not care about directory
+                # these are created based on files
+                continue
+            self._import_file_to_dotpath(src, dst)
+
+        return os.path.exists(in_dotpath_abs)
+
+    def _import_file_to_dotpath(self, src, dst):
+        self.log.dbg(f'importing {src} to {dst}')
+        try:
+            os.makedirs(os.path.dirname(dst), exist_ok=True)
+            shutil.copy2(src, dst)
+        except IOError as exc:
+            self.log.err(f'importing \"{src}\" failed: {exc}')
+            return False
         return True
 
-    def _cp(self, src, dst):
-        """the copy function for copytree"""
-        # test if must be ignored
-        if self._ignore(src):
-            return
-        shutil.copy2(src, dst)
-
     def _already_exists(self, src, dst):
         """
         test no other dotfile exists with same
@@ -299,7 +315,7 @@ class Importer:
             return True
         return False
 
-    def _apply_trans_w(self, path, trans):
+    def _apply_trans_update(self, path, trans):
         """
         apply transformation to path on filesystem)
         returns
@@ -316,6 +332,7 @@ class Importer:
             msg = f'transformation \"{trans.key}\" failed for {path}'
             self.log.err(msg)
             if os.path.exists(tmp):
+                # ignore error
                 removepath(tmp, logger=self.log)
             return None
         return tmp
