@@ -12,7 +12,12 @@ import shutil
 # local imports
 from dotdrop.logger import Logger
 from dotdrop.linktypes import LinkTypes
-from dotdrop import utils
+from dotdrop.utils import copyfile, get_file_perm, \
+    pivot_path, must_ignore, removepath, \
+    samefile, write_to_tmpfile, fastdiff, \
+    content_empty
+from dotdrop.utils import chmod as chmodit
+from dotdrop.utils import diff as diffit
 from dotdrop.exceptions import UndefinedException
 from dotdrop.cfg_yaml import CfgYaml
 
@@ -23,7 +28,8 @@ class Installer:
     def __init__(self, base='.', create=True, backup=True,
                  dry=False, safe=False, workdir='~/.config/dotdrop',
                  debug=False, diff=True, totemp=None, showdiff=False,
-                 backup_suffix='.dotdropbak', diff_cmd=''):
+                 backup_suffix='.dotdropbak', diff_cmd='',
+                 remove_existing_in_dir=False, force_chmod=False):
         """
         @base: directory path where to search for templates
         @create: create directory hierarchy if missing when installing
@@ -37,6 +43,9 @@ class Installer:
         @showdiff: show the diff before overwriting (or asking for)
         @backup_suffix: suffix for dotfile backup file
         @diff_cmd: diff command to use
+        @remove_existing_in_dir: remove file in dir dotfiles
+                                 if not managed by dotdrop
+        @force_chmod: apply chmod without confirmation
         """
         self.create = create
         self.backup = backup
@@ -55,6 +64,8 @@ class Installer:
         self.backup_suffix = backup_suffix
         self.diff_cmd = diff_cmd
         self.action_executed = False
+        self.remove_existing_in_dir = remove_existing_in_dir
+        self.force_chmod = force_chmod
         # avoids printing file copied logs
         # when using install_to_tmp for comparing
         self.comparing = False
@@ -68,7 +79,7 @@ class Installer:
     def install(self, templater, src, dst, linktype,
                 actionexec=None, noempty=False,
                 ignore=None, is_template=True,
-                chmod=None, force_chmod=False):
+                chmod=None):
         """
         install src to dst
 
@@ -81,7 +92,6 @@ class Installer:
         @ignore: pattern to ignore when installing
         @is_template: this dotfile is a template
         @chmod: rights to apply if any
-        @force_chmod: do not ask user to chmod
 
         return
         - True, None        : success
@@ -125,10 +135,13 @@ class Installer:
         if linktype == LinkTypes.NOLINK:
             # normal file
             if isdir:
-                ret, err = self._copy_dir(templater, src, dst,
-                                          actionexec=actionexec,
-                                          noempty=noempty, ignore=ignore,
-                                          is_template=is_template)
+                ret, err, ins = self._copy_dir(templater, src, dst,
+                                               actionexec=actionexec,
+                                               noempty=noempty, ignore=ignore,
+                                               is_template=is_template,
+                                               chmod=chmod)
+                if self.remove_existing_in_dir and ins:
+                    self._remove_existing_in_dir(dst, ins)
             else:
                 ret, err = self._copy_file(templater, src, dst,
                                            actionexec=actionexec,
@@ -162,7 +175,7 @@ class Installer:
                                                ignore=ignore)
 
         if self.log.debug and chmod:
-            cur = utils.get_file_perm(dst)
+            cur = get_file_perm(dst)
             if chmod == CfgYaml.chmod_ignore:
                 chmodstr = CfgYaml.chmod_ignore
             else:
@@ -175,40 +188,55 @@ class Installer:
         if self.dry:
             return self._log_install(ret, err)
 
-        # handle chmod
-        # - on success (r, not err)
-        # - no change (not r, not err)
-        # but not when
-        # - error (not r, err)
-        # - aborted (not r, err)
-        # - special keyword "preserve"
+        self._apply_chmod_after_install(src, dst, ret, err,
+                                        chmod=chmod,
+                                        linktype=linktype)
+
+        return self._log_install(ret, err)
+
+    def _apply_chmod_after_install(self, src, dst, ret, err,
+                                   chmod=None,
+                                   is_sub=False,
+                                   linktype=LinkTypes.NOLINK):
+        """
+        handle chmod after install
+        - on success (r, not err)
+        - no change (not r, not err)
+        but not when
+        - error (not r, err)
+        - aborted (not r, err)
+        - special keyword "preserve"
+        is_sub is used to specify if the file/dir is
+        part of a dotfile directory
+        """
         apply_chmod = linktype in [LinkTypes.NOLINK, LinkTypes.LINK_CHILDREN]
         apply_chmod = apply_chmod and os.path.exists(dst)
         apply_chmod = apply_chmod and (ret or (not ret and not err))
         apply_chmod = apply_chmod and chmod != CfgYaml.chmod_ignore
-        if apply_chmod:
-            if not chmod:
-                chmod = utils.get_file_perm(src)
-            self.log.dbg(f'applying chmod {chmod:o} to {dst}')
-            dstperms = utils.get_file_perm(dst)
-            if dstperms != chmod:
-                # apply mode
-                msg = f'chmod {dst} to {chmod:o}'
-                if not force_chmod and self.safe and not self.log.ask(msg):
-                    ret = False
-                    err = 'aborted'
-                else:
-                    if not self.comparing:
-                        self.log.sub(f'chmod {dst} to {chmod:o}')
-                    if utils.chmod(dst, chmod, debug=self.debug):
-                        ret = True
-                    else:
-                        ret = False
-                        err = 'chmod failed'
-        else:
+        if is_sub:
+            chmod = None
+        if not apply_chmod:
             self.log.dbg('no chmod applied')
-
-        return self._log_install(ret, err)
+            return
+        if not chmod:
+            chmod = get_file_perm(src)
+            self.log.dbg(f'dotfile in dotpath perm: {chmod:o}')
+        self.log.dbg(f'applying chmod {chmod:o} to {dst}')
+        dstperms = get_file_perm(dst)
+        if dstperms != chmod:
+            # apply mode
+            msg = f'chmod {dst} to {chmod:o}'
+            if not self.force_chmod and self.safe and not self.log.ask(msg):
+                ret = False
+                err = 'aborted'
+            else:
+                if not self.comparing:
+                    self.log.sub(f'chmod {dst} to {chmod:o}')
+                if chmodit(dst, chmod, debug=self.debug):
+                    ret = True
+                else:
+                    ret = False
+                    err = 'chmod failed'
 
     def install_to_temp(self, templater, tmpdir, src, dst,
                         is_template=True, chmod=None, ignore=None,
@@ -250,7 +278,7 @@ class Installer:
         self.totemp = None
 
         # install the dotfile to a temp directory
-        tmpdst = utils.pivot_path(dst, tmpdir, logger=self.log)
+        tmpdst = pivot_path(dst, tmpdir, logger=self.log)
         ret, err = self.install(templater, src, tmpdst,
                                 LinkTypes.NOLINK,
                                 is_template=is_template,
@@ -331,8 +359,8 @@ class Installer:
         """
         if is_template:
             self.log.dbg(f'is a template, installing to {self.workdir}')
-            tmp = utils.pivot_path(dst, self.workdir,
-                                   striphome=True, logger=self.log)
+            tmp = pivot_path(dst, self.workdir,
+                             striphome=True, logger=self.log)
             ret, err = self.install(templater, src, tmp,
                                     LinkTypes.NOLINK,
                                     actionexec=actionexec,
@@ -388,7 +416,7 @@ class Installer:
             subsrc = srcs[i]
             subdst = dsts[i]
 
-            if utils.must_ignore([subsrc, subdst], ignore, debug=self.debug):
+            if must_ignore([subsrc, subdst], ignore, debug=self.debug):
                 self.log.dbg(
                     f'ignoring install of {src} to {dst}',
                 )
@@ -399,8 +427,8 @@ class Installer:
             if is_template:
                 self.log.dbg('child is a template')
                 self.log.dbg(f'install to {self.workdir} and symlink')
-                tmp = utils.pivot_path(subdst, self.workdir,
-                                       striphome=True, logger=self.log)
+                tmp = pivot_path(subdst, self.workdir,
+                                 striphome=True, logger=self.log)
                 ret2, err2 = self.install(templater, subsrc, tmp,
                                           LinkTypes.NOLINK,
                                           actionexec=actionexec,
@@ -454,9 +482,12 @@ class Installer:
                 return False, 'aborted'
 
             # remove symlink
+            if self.backup and not os.path.isdir(dst):
+                if not self._backup(dst):
+                    return False, f'could not backup {dst}'
             overwrite = True
             try:
-                utils.removepath(dst)
+                removepath(dst)
             except OSError as exc:
                 err = f'something went wrong with {src}: {exc}'
                 return False, err
@@ -481,7 +512,7 @@ class Installer:
             if self.safe and not overwrite and not self.log.ask(msg):
                 return False, 'aborted'
             try:
-                utils.removepath(dst)
+                removepath(dst)
             except OSError as exc:
                 err = f'something went wrong with {src}: {exc}'
                 return False, err
@@ -497,7 +528,7 @@ class Installer:
         os.symlink(lnk_src, dst)
         self.log.dbg(
             f'symlink {dst} to {lnk_src} '
-            f'(mode:{utils.get_file_perm(dst):o})'
+            f'(mode:{get_file_perm(dst):o})'
         )
         if not self.comparing:
             self.log.sub(f'linked {dst} to {lnk_src}')
@@ -522,12 +553,12 @@ class Installer:
         self.log.dbg(f'no empty: {noempty}')
 
         # ignore file
-        if utils.must_ignore([src, dst], ignore, debug=self.debug):
+        if must_ignore([src, dst], ignore, debug=self.debug):
             self.log.dbg(f'ignoring install of {src} to {dst}')
             return False, None
 
         # check no loop
-        if utils.samefile(src, dst):
+        if samefile(src, dst):
             err = f'dotfile points to itself: {dst}'
             return False, err
 
@@ -540,6 +571,7 @@ class Installer:
         content = None
         if is_template:
             # template the file
+            self.log.dbg(f'it is a template: {src}')
             saved = templater.add_tmp_vars(self._get_tmp_file_vars(src, dst))
             try:
                 content = templater.generate(src)
@@ -548,7 +580,7 @@ class Installer:
             finally:
                 templater.restore_vars(saved)
             # test is empty
-            if noempty and utils.content_empty(content):
+            if noempty and content_empty(content):
                 self.log.dbg(f'ignoring empty template: {src}')
                 return False, None
             if content is None:
@@ -561,7 +593,7 @@ class Installer:
                                actionexec=actionexec)
 
         if ret and not err:
-            rights = f'{utils.get_file_perm(src):o}'
+            rights = f'{get_file_perm(src):o}'
             self.log.dbg(f'installed file {src} to {dst} ({rights})')
             if not self.dry and not self.comparing:
                 self.log.sub(f'install {src} to {dst}')
@@ -569,7 +601,8 @@ class Installer:
 
     def _copy_dir(self, templater, src, dst,
                   actionexec=None, noempty=False,
-                  ignore=None, is_template=True):
+                  ignore=None, is_template=True,
+                  chmod=None):
         """
         install src to dst when is a directory
 
@@ -578,10 +611,15 @@ class Installer:
         - False, error_msg  : error
         - False, None       : ignored
         - False, 'aborted'    : user aborted
+
+        third arg returned is the list of managed dotfiles
+        in the destination or an empty list if anything
+        fails
         """
         self.log.dbg(f'deploy dir {src}')
         # default to nothing installed and no error
-        ret = False, None
+        ret = False
+        dst_dotfiles = []
 
         # handle all files in dir
         for entry in os.listdir(src):
@@ -589,35 +627,82 @@ class Installer:
             self.log.dbg(f'deploy sub from {dst}: {entry}')
             if not os.path.isdir(fpath):
                 # is file
+                fdst = os.path.join(dst, entry)
+                dst_dotfiles.append(fdst)
                 res, err = self._copy_file(templater, fpath,
-                                           os.path.join(dst, entry),
+                                           fdst,
                                            actionexec=actionexec,
                                            noempty=noempty,
                                            ignore=ignore,
                                            is_template=is_template)
                 if not res and err:
                     # error occured
-                    return res, err
+                    return res, err, []
+
+                self._apply_chmod_after_install(fpath, fdst, ret, err,
+                                                chmod=chmod, is_sub=True)
 
                 if res:
                     # something got installed
-                    ret = True, None
+
+                    ret = True
             else:
                 # is directory
-                res, err = self._copy_dir(templater, fpath,
-                                          os.path.join(dst, entry),
-                                          actionexec=actionexec,
-                                          noempty=noempty,
-                                          ignore=ignore,
-                                          is_template=is_template)
+                dpath = os.path.join(dst, entry)
+                dst_dotfiles.append(dpath)
+                res, err, subs = self._copy_dir(templater, fpath,
+                                                dpath,
+                                                actionexec=actionexec,
+                                                noempty=noempty,
+                                                ignore=ignore,
+                                                is_template=is_template)
+                dst_dotfiles.extend(subs)
                 if not res and err:
                     # error occured
-                    return res, err
+                    return res, err, []
 
                 if res:
                     # something got installed
-                    ret = True, None
-        return ret
+                    ret = True
+
+        return ret, None, dst_dotfiles
+
+    def _is_path_in(self, path, paths):
+        """return true if path is in paths"""
+        return any(samefile(path, p) for p in paths)
+
+    def _remove_existing_in_dir(self, directory, installed_files=None):
+        """
+        with --remove-existing this will remove
+        any file in managed directory which
+        are not handled by dotdrop
+        """
+        if not installed_files:
+            return
+        if not os.path.exists(directory) or not os.path.isdir(directory):
+            return
+        to_remove = []
+        for root, dirs, files in os.walk(directory):
+            for name in files:
+                path = os.path.join(root, name)
+                if self._is_path_in(path, installed_files):
+                    continue
+                to_remove.append(os.path.abspath(path))
+            for name in dirs:
+                path = os.path.join(root, name)
+                if self._is_path_in(path, installed_files):
+                    continue
+                to_remove.append(os.path.abspath(path))
+        for path in to_remove:
+            if self.dry:
+                self.log.dry(f'would remove {path}')
+                continue
+            if self.safe:
+                if not self.log.ask(f'remove unmanaged \"{path}\"'):
+                    return
+            self.log.dbg(f'removing unmanaged file \"{path}\"')
+            if not removepath(path, logger=self.log):
+                self.log.warn(f'unable to remove {path}')
 
     @classmethod
     def _write_content_to_file(cls, content, src, dst):
@@ -627,7 +712,6 @@ class Installer:
             try:
                 with open(dst, 'wb') as file:
                     file.write(content)
-                # shutil.copymode(src, dst)
             except NotADirectoryError as exc:
                 err = f'opening dest file: {exc}'
                 return False, err
@@ -638,8 +722,8 @@ class Installer:
         else:
             # copy file
             try:
+                # do NOT copy meta here
                 shutil.copyfile(src, dst)
-                # shutil.copymode(src, dst)
             except OSError as exc:
                 return False, str(exc)
         return True, None
@@ -662,6 +746,7 @@ class Installer:
 
         if os.path.lexists(dst):
             # file/symlink exists
+            self.log.dbg(f'file already exists on filesystem: {dst}')
             try:
                 os.stat(dst)
             except OSError as exc:
@@ -686,7 +771,10 @@ class Installer:
                 overwrite = True
 
             if self.backup:
-                self._backup(dst)
+                if not self._backup(dst):
+                    return False, f'could not backup {dst}'
+        else:
+            self.log.dbg(f'file does not exist on filesystem: {dst}')
 
         # create hierarchy
         base = os.path.dirname(dst)
@@ -708,9 +796,9 @@ class Installer:
             return False, 'aborted'
 
         # writing to file
-        self.log.dbg(f'before writing to {dst} ({utils.get_file_perm(src):o})')
+        self.log.dbg(f'before writing to {dst} ({get_file_perm(src):o})')
         ret = self._write_content_to_file(content, src, dst)
-        self.log.dbg(f'written to {dst} ({utils.get_file_perm(src):o})')
+        self.log.dbg(f'written to {dst} ({get_file_perm(src):o})')
         return ret
 
     ########################################################
@@ -732,13 +820,14 @@ class Installer:
         # check file content
         tmp = None
         if content:
-            tmp = utils.write_to_tmpfile(content)
+            tmp = write_to_tmpfile(content)
             src = tmp
-        ret = utils.fastdiff(src, dst)
+        ret = fastdiff(src, dst)
         if ret:
             self.log.dbg('content differ')
         if content:
-            utils.removepath(tmp)
+            # ignore error
+            removepath(tmp)
         return ret
 
     def _show_diff_before_write(self, src, dst, content=None):
@@ -749,12 +838,13 @@ class Installer:
         """
         tmp = None
         if content:
-            tmp = utils.write_to_tmpfile(content)
+            tmp = write_to_tmpfile(content)
             src = tmp
-        diff = utils.diff(modified=src, original=dst,
-                          diff_cmd=self.diff_cmd)
+        diff = diffit(modified=src, original=dst,
+                      diff_cmd=self.diff_cmd)
         if tmp:
-            utils.removepath(tmp, logger=self.log)
+            # ignore error
+            removepath(tmp, logger=self.log)
 
         if diff:
             self._print_diff(src, dst, diff)
@@ -783,16 +873,20 @@ class Installer:
     def _backup(self, path):
         """backup file pointed by path"""
         if self.dry:
-            return
+            return True
         dst = path.rstrip(os.sep) + self.backup_suffix
         self.log.log(f'backup {path} to {dst}')
         # os.rename(path, dst)
         # copy to preserve mode on chmod=preserve
         # since we expect dotfiles this shouldn't have
         # such a big impact but who knows.
-        shutil.copy2(path, dst)
+        if not copyfile(path, dst, debug=self.debug):
+            return False
+        if not os.path.exists(dst):
+            return False
         stat = os.stat(path)
         os.chown(dst, stat.st_uid, stat.st_gid)
+        return True
 
     def _exec_pre_actions(self, actionexec):
         """execute action executor"""
