@@ -21,6 +21,7 @@ from dotdrop.cfg_aggregator import CfgAggregator
 from dotdrop.action import Action
 from dotdrop.utils import uniq_list, debug_list, debug_dict
 from dotdrop.exceptions import YamlException, OptionsException
+from dotdrop.config import DEFAULT_CONFIG
 
 ENV_PROFILE = 'DOTDROP_PROFILE'
 ENV_CONFIG = 'DOTDROP_CONFIG'
@@ -58,19 +59,22 @@ USAGE = f"""
 {BANNER}
 
 Usage:
-  dotdrop install   [-VbtfndDaW] [-c <path>] [-p <profile>]
-                                 [-w <nb>] [<key>...]
-  dotdrop import    [-Vbdfm]     [-c <path>] [-p <profile>] [-i <pattern>...]
-                                 [--transr=<key>] [--transw=<key>]
-                                 [-l <link>] [-s <path>] <path>...
-  dotdrop compare   [-LVbz]      [-c <path>] [-p <profile>]
-                                 [-w <nb>] [-C <file>...] [-i <pattern>...]
-  dotdrop update    [-VbfdkPz]   [-c <path>] [-p <profile>]
-                                 [-w <nb>] [-i <pattern>...] [<path>...]
-  dotdrop remove    [-Vbfdk]     [-c <path>] [-p <profile>] [<path>...]
-  dotdrop files     [-VbTG]      [-c <path>] [-p <profile>]
-  dotdrop detail    [-Vb]        [-c <path>] [-p <profile>] [<key>...]
-  dotdrop profiles  [-VbG]       [-c <path>]
+  dotdrop install   [-VbtfndDaWR] [-c <path>] [-p <profile>]
+                                  [-w <nb>] [<key>...]
+  dotdrop import    [-Vbdfm]      [-c <path>] [-p <profile>]
+                                  [-i <pattern>...] [--dkey=<key>]
+                                  [--transr=<key>] [--transw=<key>]
+                                  [-l <link>] [-s <path>] <path>...
+  dotdrop compare   [-LVbz]       [-c <path>] [-p <profile>]
+                                  [-w <nb>] [-C <file>...] [-i <pattern>...]
+  dotdrop update    [-VbfdkPz]    [-c <path>] [-p <profile>]
+                                  [-w <nb>] [-i <pattern>...] [<path>...]
+  dotdrop remove    [-Vbfdk]      [-c <path>] [-p <profile>] [<path>...]
+  dotdrop uninstall [-Vbfd]       [-c <path>] [-p <profile>] [<key>...]
+  dotdrop files     [-VbTG]       [-c <path>] [-p <profile>]
+  dotdrop detail    [-Vb]         [-c <path>] [-p <profile>] [<key>...]
+  dotdrop profiles  [-VbG]        [-c <path>]
+  dotdrop gencfg
   dotdrop --help
   dotdrop --version
 
@@ -85,15 +89,17 @@ Options:
   -G --grepable           Grepable output.
   -i --ignore=<pattern>   Pattern to ignore.
   -k --key                Treat <path> as a dotfile key.
+  -K --dkey=<key>         Set the dotfile key.
   -l --link=<link>        Link option (nolink|absolute|relative|link_children).
   -L --file-only          Do not show diff but only the files that differ.
   -m --preserve-mode      Insert a chmod entry in the dotfile with its mode.
   -n --nodiff             Do not diff when installing.
   -p --profile=<profile>  Specify the profile to use [default: {PROFILE}].
   -P --show-patch         Provide a one-liner to manually patch template.
+  -R --remove-existing    Remove existing file on install directory.
   -s --as=<path>          Import as a different path from actual path.
-  --transr=<key>          Associate trans_read key on import.
-  --transw=<key>          Apply trans_write key on import.
+  --transr=<key>          Associate trans_install key on import.
+  --transw=<key>          Apply trans_update key on import.
   -t --temp               Install to a temporary directory for review.
   -T --template           Only template dotfiles.
   -V --verbose            Be verbose.
@@ -144,10 +150,16 @@ class Options(AttrMonitor):
 
         # args parsing
         self.args = {}
-        if not args:
-            self.args = docopt(USAGE, version=VERSION)
         if args:
             self.args = args.copy()
+        else:
+            self.args = docopt(USAGE, version=VERSION)
+
+        if self.args['gencfg']:
+            # print config and exit
+            print(DEFAULT_CONFIG)
+            sys.exit(0)
+
         self.debug = self.args['--verbose'] or ENV_DEBUG in os.environ
         self.log = Logger(debug=self.debug)
         self.dry = self.args['--dry']
@@ -157,19 +169,21 @@ class Options(AttrMonitor):
         # selected profile
         self.profile = self.args['--profile']
         self.confpath = self._get_config_path()
+        if not self.confpath:
+            raise YamlException('no config file found')
         self.confpath = os.path.abspath(self.confpath)
         self.log.dbg(f'config abs path: {self.confpath}')
         if not self.confpath:
             raise YamlException('no config file found')
         if not os.path.exists(self.confpath):
-            err = f'bad config file path: {self.confpath}'
+            err = f'config does not exist \"{self.confpath}\"'
             raise YamlException(err)
         self.log.dbg('#################################################')
         self.log.dbg('#################### DOTDROP ####################')
         self.log.dbg('#################################################')
         self.log.dbg(f'version: {VERSION}')
-        args = ' '.join(sys.argv)
-        self.log.dbg(f'command: {args}')
+        self.argv = ' '.join(sys.argv)
+        self.log.dbg(f'command: {self.argv}')
         self.log.dbg(f'config file: {self.confpath}')
 
         self._read_config()
@@ -183,6 +197,10 @@ class Options(AttrMonitor):
         # start monitoring for bad attribute
         self._set_attr_err = True
 
+    def debug_command(self):
+        """print the original command"""
+        self.log.dbg(f'command: {self.argv}')
+
     @classmethod
     def _get_config_from_env(cls, name):
         # look in XDG_CONFIG_HOME
@@ -297,6 +315,7 @@ class Options(AttrMonitor):
         self.install_force_chmod = self.force_chmod
         self.install_clear_workdir = self.args['--workdir-clear'] or \
             self.clear_workdir
+        self.install_remove_existing = self.args['--remove-existing']
 
     def _apply_args_compare(self):
         """compare specifics"""
@@ -318,8 +337,9 @@ class Options(AttrMonitor):
         self.import_ignore.extend(self.impignore)
         self.import_ignore.append(f'*{self.install_backup_suffix}')
         self.import_ignore = uniq_list(self.import_ignore)
-        self.import_transw = self.args['--transw']
-        self.import_transr = self.args['--transr']
+        self.import_trans_install = self.args['--transr']
+        self.import_trans_update = self.args['--transw']
+        self.import_force_key = self.args['--dkey']
 
     def _apply_args_update(self):
         """update specifics"""
@@ -340,6 +360,10 @@ class Options(AttrMonitor):
         self.remove_path = self.args['<path>']
         self.remove_iskey = self.args['--key']
 
+    def _apply_args_uninstall(self):
+        """uninstall specifics"""
+        self.uninstall_key = self.args['<key>']
+
     def _apply_args_detail(self):
         """detail specifics"""
         self.detail_keys = self.args['<key>']
@@ -355,6 +379,7 @@ class Options(AttrMonitor):
         self.cmd_update = self.args['update']
         self.cmd_detail = self.args['detail']
         self.cmd_remove = self.args['remove']
+        self.cmd_uninstall = self.args['uninstall']
 
         # adapt attributes based on arguments
         self.safe = not self.args['--force']
@@ -403,6 +428,9 @@ class Options(AttrMonitor):
         # "remove" specifics
         self._apply_args_remove()
 
+        # "uninstall" specifics
+        self._apply_args_uninstall()
+
     def _fill_attr(self):
         """create attributes from conf"""
         # defined variables
