@@ -9,6 +9,7 @@ import os
 import io
 import re
 import mmap
+import sys
 from jinja2 import Environment, FileSystemLoader, \
     ChoiceLoader, FunctionLoader, TemplateNotFound, \
     StrictUndefined
@@ -29,6 +30,9 @@ COMMENT_START = '{#@@'
 COMMENT_END = '@@#}'
 LOG = Logger()
 
+DICT_ENV_NAME = 'env'
+DICT_VARS_NAME = '_vars'
+
 
 class Templategen:
     """dotfile templater"""
@@ -62,9 +66,10 @@ class Templategen:
                                undefined=StrictUndefined)
 
         # adding variables
-        self.variables['env'] = os.environ
+        self.variables[DICT_ENV_NAME] = os.environ
         if variables:
             self.variables.update(variables)
+            self.variables[DICT_VARS_NAME] = variables
 
         # adding header method
         self.env.globals['header'] = self._header
@@ -110,6 +115,38 @@ class Templategen:
             err = f'undefined variable: {exc.message}'
             raise UndefinedException(err) from exc
 
+    def generate_dict(self, dic):
+        """
+        template each entry of the dict where only
+        the value of each entry is templated (recursively)
+        may raise a UndefinedException
+        in case a variable is undefined
+        """
+        if not dic:
+            return dic
+        for key in dic:
+            value = dic[key]
+            if isinstance(value, str):
+                dic[key] = self.generate_string(value)
+                continue
+            if isinstance(value, dict):
+                dic[key] = self.generate_dict(value)
+                continue
+            continue
+        return dic
+
+    def generate_string_or_dict(self, content):
+        """
+        render template from string or dict
+        may raise a UndefinedException
+        in case a variable is undefined
+        """
+        if isinstance(content, str):
+            return self.generate_string(content)
+        if isinstance(content, dict):
+            return self.generate_dict(content)
+        raise UndefinedException(f'could not template {content}')
+
     def add_tmp_vars(self, newvars=None):
         """add vars to the globals, make sure to call restore_vars"""
         saved_variables = self.variables.copy()
@@ -147,8 +184,8 @@ class Templategen:
         """add a comment usually in the header of a dotfile"""
         return f'{prepend}{utils.header()}'
 
-    def _handle_file(self, src):
-        """generate the file content from template"""
+    def _get_filetype(self, src):
+        """use magic or the file command to get the mime type of a file"""
         try:
             # pylint: disable=C0415
             import magic
@@ -156,10 +193,27 @@ class Templategen:
             self.log.dbg('using \"magic\" for filetype identification')
         except ImportError:
             # fallback
-            _, filetype = utils.run(['file', '-b', '--mime-type', src],
+
+            # `file` on Windows doesn't support `-L`
+            follow_symlink = ['-L'] if sys.platform != 'win32' else []
+
+            _, filetype = utils.run(['file'] + follow_symlink + ['-b',
+                                                                 '--mime-type',
+                                                                 src],
                                     debug=self.debug)
             self.log.dbg('using \"file\" for filetype identification')
             filetype = filetype.strip()
+        if filetype == 'inode/symlink':
+            dst = os.readlink(src)
+            if dst[0] != '/':
+                # Canonicalize relative path
+                dst = os.path.join(os.path.dirname(src), dst)
+            return self._get_filetype(dst)
+        return filetype
+
+    def _handle_file(self, src):
+        """generate the file content from template"""
+        filetype = self._get_filetype(src)
         istext = self._is_text(filetype)
         self.log.dbg(f'filetype \"{src}\": {filetype}')
         self.log.dbg(f'is text \"{src}\": {istext}')
@@ -177,6 +231,10 @@ class Templategen:
             return True
         if 'json' in out:
             return True
+        if 'javascript' in out:
+            return True
+        if 'ecmascript' in out:
+            return True
         return False
 
     def _template_loader(self, relpath):
@@ -217,47 +275,73 @@ class Templategen:
         return data.decode('utf-8', 'replace')
 
     @staticmethod
-    def is_template(path, ignore=None, debug=False):
+    def path_is_template(path, debug=False):
         """recursively check if any file is a template within path"""
-        if debug:
-            LOG.dbg(f'is template: {path}', force=True)
         path = os.path.expanduser(path)
 
         if not os.path.exists(path):
             # does not exist
-            return False
-
-        if utils.must_ignore([path], ignore, debug=debug):
-            # must be ignored
+            if debug:
+                LOG.dbg(f'is NOT template: \"{path}\"', force=True)
             return False
 
         if os.path.isfile(path):
             # is file
-            return Templategen._is_template(path, ignore=ignore, debug=debug)
+            return Templategen._is_template(path, debug=debug)
 
         # is a directory
         for entry in os.listdir(path):
             fpath = os.path.join(path, entry)
             if not os.path.isfile(fpath):
                 # recursively explore directory
-                if Templategen.is_template(fpath, ignore=ignore, debug=debug):
+                if Templategen.path_is_template(fpath,
+                                                debug=debug):
+                    if debug:
+                        LOG.dbg(f'is indeed template: \"{path}\"', force=True)
                     return True
             else:
                 # check if file is a template
-                if Templategen._is_template(fpath, ignore=ignore, debug=debug):
+                if Templategen._is_template(fpath,
+                                            debug=debug):
+                    if debug:
+                        LOG.dbg(f'is indeed template: \"{path}\"', force=True)
                     return True
+        if debug:
+            LOG.dbg(f'is NOT template: \"{path}\"', force=True)
         return False
 
     @staticmethod
-    def var_is_template(string):
+    def string_is_template(string):
         """check if variable contains template(s)"""
         return VAR_START in str(string)
 
     @staticmethod
-    def _is_template(path, ignore, debug=False):
+    def dict_is_template(dic):
+        """check if dict contains template(s)"""
+        for key in dic:
+            value = dic[key]
+            if isinstance(value, str):
+                isit = Templategen.string_is_template(value)
+                if isit:
+                    return True
+            elif isinstance(value, dict):
+                return Templategen.dict_is_template(value)
+        return False
+
+    @staticmethod
+    def var_is_template(something):
+        """check if string or dict is template"""
+        if isinstance(something, str):
+            return Templategen.string_is_template(something)
+        if isinstance(something, dict):
+            return Templategen.dict_is_template(something)
+        return False
+
+    @staticmethod
+    def _is_template(path, debug=False):
         """test if file pointed by path is a template"""
-        if utils.must_ignore([path], ignore, debug=debug):
-            return False
+        if debug:
+            Logger().dbg(f'is template: {path}')
         if not os.path.isfile(path):
             return False
         if os.stat(path).st_size == 0:
