@@ -8,13 +8,14 @@ handle the update of dotfiles
 import os
 import shutil
 import filecmp
-import fnmatch
 
 # local imports
 from dotdrop.logger import Logger
+from dotdrop.ftree import FTreeDir
 from dotdrop.templategen import Templategen
-from dotdrop.utils import patch_ignores, removepath, get_unique_tmp_name, \
-    write_to_tmpfile, must_ignore, mirror_file_rights, get_file_perm
+from dotdrop.utils import ignores_to_absolute, removepath, \
+    get_unique_tmp_name, write_to_tmpfile, must_ignore, \
+    mirror_file_rights, get_file_perm, diff
 from dotdrop.exceptions import UndefinedException
 
 
@@ -47,7 +48,6 @@ class Updater:
         self.safe = safe
         self.debug = debug
         self.ignore = ignore or []
-        self.ignores = None
         self.showpatch = showpatch
         self.ignore_missing_in_dotdrop = ignore_missing_in_dotdrop
         self.templater = Templategen(variables=self.variables,
@@ -96,8 +96,10 @@ class Updater:
         ret = False
         new_path = None
         ignores = list(set(self.ignore + dotfile.upignore))
-        self.ignores = patch_ignores(ignores, dotfile.dst, debug=self.debug)
-        self.log.dbg(f'ignore pattern(s): {self.ignores}')
+        prefixes = [dotfile.dst, dotfile.src]
+        ignores = ignores_to_absolute(ignores, prefixes,
+                                      debug=self.debug)
+        self.log.dbg(f'ignore pattern(s) for {path}: {ignores}')
 
         deployed_path = os.path.expanduser(path)
         local_path = os.path.join(self.dotpath, dotfile.src)
@@ -115,12 +117,13 @@ class Updater:
 
         ignore_missing_in_dotdrop = self.ignore_missing_in_dotdrop or \
             dotfile.ignore_missing_in_dotdrop
-        if (ignore_missing_in_dotdrop and not os.path.exists(local_path)) or \
-                self._ignore([deployed_path, local_path]):
+
+        if ignore_missing_in_dotdrop and not os.path.exists(local_path):
             self.log.sub(f'\"{dotfile.key}\" ignored')
             return True
+
         # apply write transformation if any
-        new_path = self._apply_trans_w(deployed_path, dotfile)
+        new_path = self._apply_trans_update(deployed_path, dotfile)
         if not new_path:
             return False
 
@@ -130,25 +133,31 @@ class Updater:
 
         # handle the pointed file
         if os.path.isdir(new_path):
-            ret = self._handle_dir(new_path, local_path, dotfile)
+            ret = self._handle_dir(new_path, local_path,
+                                   dotfile, ignores)
         else:
-            ret = self._handle_file(new_path, local_path)
+            ret = self._handle_file(new_path, local_path,
+                                    ignores)
+        if not ret:
+            return False
 
+        # mirror rights
         if deployed_mode != local_mode:
-            # mirror rights
             msg = f'adopt mode {deployed_mode:o} for {dotfile.key}'
             self.log.dbg(msg)
             if self.conf.update_dotfile(dotfile.key, deployed_mode):
                 ret = True
+            self._mirror_file_perms(deployed_path, local_path)
 
         # clean temporary files
         if new_path != deployed_path and os.path.exists(new_path):
+            # ignore error
             removepath(new_path, logger=self.log)
         return ret
 
-    def _apply_trans_w(self, path, dotfile):
+    def _apply_trans_update(self, path, dotfile):
         """apply write transformation to dotfile"""
-        trans = dotfile.get_trans_w()
+        trans = dotfile.get_trans_update()
         if not trans:
             return path
         self.log.dbg(f'executing write transformation {trans}')
@@ -159,6 +168,7 @@ class Updater:
         if not trans.transform(path, tmp, templater=self.templater,
                                debug=self.debug):
             if os.path.exists(tmp):
+                # ignore error
                 removepath(tmp, logger=self.log)
             err = f'transformation \"{trans.key}\" failed for {dotfile.key}'
             self.log.err(err)
@@ -166,8 +176,8 @@ class Updater:
         return tmp
 
     def _is_template(self, path):
-        if not Templategen.is_template(path, ignore=self.ignores,
-                                       debug=self.debug):
+        if not Templategen.path_is_template(path,
+                                            debug=self.debug):
             self.log.dbg(f'{path} is NO template')
             return False
         self.log.warn(f'{path} uses template, update manually')
@@ -198,7 +208,7 @@ class Updater:
             self.log.err(exc)
             return False
 
-    def _mirror_rights(self, src, dst):
+    def _mirror_file_perms(self, src, dst):
         srcr = get_file_perm(src)
         dstr = get_file_perm(dst)
         if srcr == dstr:
@@ -210,9 +220,10 @@ class Updater:
         except OSError as exc:
             self.log.err(exc)
 
-    def _handle_file(self, deployed_path, local_path, compare=True):
+    def _handle_file(self, deployed_path, local_path,
+                     ignores, compare=True):
         """sync path (deployed file) and local_path (dotdrop dotfile path)"""
-        if self._ignore([deployed_path, local_path]):
+        if self._must_ignore([deployed_path, local_path], ignores):
             self.log.sub(f'\"{local_path}\" ignored')
             return True
         self.log.dbg(f'update for file {deployed_path} and {local_path}')
@@ -239,184 +250,100 @@ class Updater:
                 self.log.dry(f'would cp {deployed_path} {local_path}')
             else:
                 self.log.dbg(f'cp {deployed_path} {local_path}')
-                shutil.copyfile(deployed_path, local_path)
-                self._mirror_rights(deployed_path, local_path)
+                shutil.copy2(deployed_path, local_path)
+                # self._mirror_file_perms(deployed_path, local_path)
                 self.log.sub(f'\"{local_path}\" updated')
         except IOError as exc:
             self.log.warn(f'{deployed_path} update failed, do manually: {exc}')
             return False
         return True
 
-    def _handle_dir(self, deployed_path, local_path, dotfile):
+    def _handle_dir(self, deployed_path, local_path,
+                    dotfile, ignores):
         """sync path (local dir) and local_path (dotdrop dir path)"""
+        ret = True
         self.log.dbg(f'handle update for dir {deployed_path} to {local_path}')
-        # paths must be absolute (no tildes)
+
+        # get absolute paths
         deployed_path = os.path.expanduser(deployed_path)
         local_path = os.path.expanduser(local_path)
 
-        if self._ignore([deployed_path, local_path]):
-            self.log.sub(f'\"{local_path}\" ignored')
-            return True
-        # find the differences
-        diff = filecmp.dircmp(deployed_path, local_path, ignore=None)
-        # handle directories diff
-        ret = self._merge_dirs(diff, dotfile)
-        self._mirror_rights(deployed_path, local_path)
-        return ret
+        local_tree = FTreeDir(local_path,
+                              ignores=ignores,
+                              debug=self.debug)
+        deploy_tree = FTreeDir(deployed_path,
+                               ignores=ignores,
+                               debug=self.debug)
+        lonly, ronly, common = local_tree.compare(deploy_tree)
 
-    def _merge_dirs_create_left_only(self, diff, left, right,
-                                     ignore_missing_in_dotdrop):
-        """create dirs that don't exist in dotdrop"""
-        for toadd in diff.left_only:
-            exist = os.path.join(left, toadd)
-            if not os.path.isdir(exist):
-                # ignore files for now
-                continue
-            # match to dotdrop dotpath
-            new = os.path.join(right, toadd)
-            if (ignore_missing_in_dotdrop and not os.path.exists(new)) or \
-                    self._ignore([exist, new]):
-                self.log.sub(f'\"{exist}\" ignored')
-                continue
+        # those only in dotpath
+        for i in lonly:
+            path = os.path.join(local_path, i)
             if self.dry:
-                self.log.dry(f'would cp -r {exist} {new}')
+                self.log.dry(f'would rm -r {path}')
                 continue
-            self.log.dbg(f'cp -r {exist} {new}')
-
-            # Newly created directory should be copied as is (for efficiency).
-            def ign(src, names):
-                whitelist, blacklist = set(), set()
-                for ignore in self.ignores:
-                    for name in names:
-                        path = os.path.join(src, name)
-                        if ignore.startswith('!') and \
-                                fnmatch.fnmatch(path, ignore[1:]):
-                            # add to whitelist
-                            whitelist.add(name)
-                        elif fnmatch.fnmatch(path, ignore):
-                            # add to blacklist
-                            blacklist.add(name)
-                return blacklist - whitelist
-
-            try:
-                shutil.copytree(exist, new, ignore=ign)
-            except OSError as exc:
-                msg = f'error copying dir {exist}'
-                self.log.err(f'{msg}: {exc}')
+            self.log.dbg(f'rm -r {path}')
+            if not self._confirm_rm_r(path):
                 continue
-            self.log.sub(f'\"{new}\" dir added')
-
-    def _merge_dirs_remove_right_only(self, diff, left, right,
-                                      ignore_missing_in_dotdrop):
-        """remove dirs that don't exist in deployed version"""
-        for toremove in diff.right_only:
-            old = os.path.join(right, toremove)
-            if not os.path.isdir(old):
-                # ignore files for now
+            if not removepath(path, logger=self.log):
+                msg = f'unable to remove {path}, do manually'
+                self.log.warn(msg)
+                ret = False
                 continue
-            if self._ignore([old]):
-                continue
-            if self.dry:
-                self.log.dry(f'would rm -r {old}')
-                continue
-            self.log.dbg(f'rm -r {old}')
-            if not self._confirm_rm_r(old):
-                continue
-            removepath(old, logger=self.log)
-            self.log.sub(f'\"{old}\" dir removed')
-
-        # handle files diff
-        # sync files that exist in both but are different
-        fdiff = diff.diff_files
-        fdiff.extend(diff.funny_files)
-        fdiff.extend(diff.common_funny)
-        for file in fdiff:
-            fleft = os.path.join(left, file)
-            fright = os.path.join(right, file)
-            if (ignore_missing_in_dotdrop and not os.path.exists(fright)) or \
-                    self._ignore([fleft, fright]):
-                continue
-            if self.dry:
-                self.log.dry(f'would cp {fleft} {fright}')
-                continue
-            self.log.dbg(f'cp {fleft} {fright}')
-            self._handle_file(fleft, fright, compare=False)
-
-    def _merge_dirs_copy_left_only(self, diff, left, right,
-                                   ignore_missing_in_dotdrop):
-        """copy files that don't exist in dotdrop"""
-        for toadd in diff.left_only:
-            exist = os.path.join(left, toadd)
-            if os.path.isdir(exist):
-                # ignore dirs, done above
-                continue
-            new = os.path.join(right, toadd)
-            if (ignore_missing_in_dotdrop and not os.path.exists(new)) or \
-                    self._ignore([exist, new]):
-                continue
-            if self.dry:
-                self.log.dry(f'would cp {exist} {new}')
-                continue
-            self.log.dbg(f'cp {exist} {new}')
-            try:
-                shutil.copyfile(exist, new)
-            except OSError as exc:
-                msg = f'error copying file {exist}'
-                self.log.err(f'{msg}: {exc}')
-                continue
-
-            self._mirror_rights(exist, new)
-            self.log.sub(f'\"{new}\" added')
-
-    def _merge_dirs_remove_right_only_2(self, diff, right):
-        """remove files that don't exist in deployed version"""
-        for toremove in diff.right_only:
-            new = os.path.join(right, toremove)
-            if not os.path.exists(new):
-                continue
-            if os.path.isdir(new):
-                # ignore dirs, done above
-                continue
-            if self._ignore([new]):
-                continue
-            if self.dry:
-                self.log.dry(f'would rm {new}')
-                continue
-            self.log.dbg(f'rm {new}')
-            removepath(new, logger=self.log)
-            self.log.sub(f'\"{new}\" removed')
-
-    def _merge_dirs(self, diff, dotfile):
-        """Synchronize directories recursively."""
-        left, right = diff.left, diff.right
-        self.log.dbg(f'sync dir {left} to {right}')
-        if self._ignore([left, right]):
-            return True
+            self.log.sub(f'\"{path}\" removed')
 
         ignore_missing_in_dotdrop = self.ignore_missing_in_dotdrop or \
             dotfile.ignore_missing_in_dotdrop
+        if not ignore_missing_in_dotdrop:
+            for i in ronly:
+                # only in deployed dir
+                srcpath = os.path.join(deployed_path, i)
+                dstpath = os.path.join(local_path, i)
+                if self.dry:
+                    self.log.dry(f'would cp -r {srcpath} {dstpath}')
+                    continue
+                self.log.dbg(f'cp {srcpath} {dstpath}')
+                try:
+                    if not os.path.isdir(srcpath):
+                        # we do not care about directory
+                        os.makedirs(os.path.dirname(dstpath), exist_ok=True)
+                        shutil.copy2(srcpath, dstpath)
+                    # self._mirror_file_perms(srcpath, dstpath)
+                except IOError as exc:
+                    msg = f'{srcpath} update right only failed'
+                    msg += f', do manually: {exc}'
+                    self.log.warn(msg)
+                    ret = False
+                    continue
+                self.log.sub(f'\"{dstpath}\" updated')
 
-        self._merge_dirs_create_left_only(diff, left, right,
-                                          ignore_missing_in_dotdrop)
-        self._merge_dirs_remove_right_only(diff, left, right,
-                                           ignore_missing_in_dotdrop)
-        self._merge_dirs_copy_left_only(diff, left, right,
-                                        ignore_missing_in_dotdrop)
-        self._merge_dirs_remove_right_only_2(diff, right)
-
-        # compare rights
-        for common in diff.common_files:
-            leftf = os.path.join(left, common)
-            rightf = os.path.join(right, common)
-            if not self._same_rights(leftf, rightf):
-                self._mirror_rights(leftf, rightf)
-
-        # Recursively decent into common subdirectories.
-        for subdir in diff.subdirs.values():
-            self._merge_dirs(subdir, dotfile)
-
-        # Nothing more to do here.
-        return True
+        for i in common:
+            srcpath = os.path.join(deployed_path, i)
+            dstpath = os.path.join(local_path, i)
+            if os.path.isdir(srcpath):
+                continue
+            if not self._same_rights(dstpath, srcpath):
+                # update rights
+                self._mirror_file_perms(srcpath, dstpath)
+            out = diff(modified=dstpath, original=srcpath,
+                       debug=self.debug)
+            if not out:
+                continue
+            if self.dry:
+                msg = f'would update content of {dstpath} from {srcpath}'
+                self.log.dry(msg)
+                continue
+            self.log.dbg(f'cp {srcpath} {dstpath}')
+            try:
+                shutil.copy2(srcpath, dstpath)
+                self._mirror_file_perms(srcpath, dstpath)
+            except IOError as exc:
+                msg = f'{srcpath} update common failed, do manually: {exc}'
+                self.log.warn(msg)
+                ret = False
+                continue
+            self.log.sub(f'\"{dstpath}\" content updated')
+        return ret
 
     def _overwrite(self, src, dst):
         """ask for overwritting"""
@@ -432,8 +359,8 @@ class Updater:
             return False
         return True
 
-    def _ignore(self, paths):
-        if must_ignore(paths, self.ignores, debug=self.debug):
+    def _must_ignore(self, paths, ignores):
+        if must_ignore(paths, ignores, debug=self.debug):
             self.log.dbg(f'ignoring update for {paths}')
             return True
         return False
