@@ -1,4 +1,4 @@
-#!/bin/bash
+#!/usr/bin/env bash
 # author: deadc0de6
 #
 # update packages
@@ -8,26 +8,17 @@
 up()
 {
   # update pkgver
-  [ "${1}" != "" ] && sed -i "s/^pkgver=.*$/pkgver=${1}/g" ${pkgfile}
+  [ "${1}" != "" ] && sed -i "s/^pkgver=.*$/pkgver=${1}/g" "${pkgfile}"
   # create srcinfo
   rm -f .SRCINFO
   makepkg --printsrcinfo > .SRCINFO
 }
 
 # pivot
-rl="readlink -f"
-if ! ${rl} "${0}" >/dev/null 2>&1; then
-  rl="realpath"
-
-  if ! hash ${rl}; then
-    echo "\"${rl}\" not found!" && exit 1
-  fi
-fi
-# cur
-cur=`dirname $(${rl} "${0}")`
-opwd=`pwd`
+cur=$(cd "$(dirname "${0}")" && pwd)
+opwd=$(pwd)
 pkgfile="PKGBUILD"
-cd ${cur}
+cd "${cur}" || exit 1
 
 ########################
 # update arch package
@@ -35,10 +26,10 @@ cd ${cur}
 ########################
 dir="arch-dotdrop"
 echo "doing ${dir} ..."
-cd ${dir}
-version="`git describe --abbrev=0 --tags | sed 's/^v//g'`"
-up ${version}
-cd ${OLDPWD}
+cd ${dir} || exit 1
+version="$(git describe --abbrev=0 --tags | sed 's/^v//g')"
+up "${version}"
+cd "${OLDPWD}" || exit 1
 
 #########################
 ## update arch package
@@ -53,4 +44,4 @@ cd ${OLDPWD}
 #cd ${OLDPWD}
 
 # pivot back
-cd ${opwd}
+cd "${opwd}" || exit
